/*-------------------------------------------------
   Enter two vectors and calculate the dot product
   using a function.
  -------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "dp.h"

/*--------------------------
  main
  --------------------------*/
int main(int argc, char *argv[]){
  int i, n;
  REAL c;
  FILE *fp;
  struct rvec a, b;

  /*--------------------------------------------
     check number of command-line arguments and
     extract length of vectors
    --------------------------------------------*/
  if(argc != 2){
    printf("Usage:\n");
    printf("  dotprod n\n");
    printf("  where n is the vector length\n");
    return 1;
  }
  n = atoi(argv[1]);

  /*-------------------------------------------
     use malloc to dynamically allocate the
     two vectors now that we know their length
    -------------------------------------------*/
  a.x = (REAL *) malloc(n*sizeof(REAL));
  b.x = (REAL *) malloc(n*sizeof(REAL));

  /*----------------------------------------
     enter two vectors
    ----------------------------------------*/
  printf("Enter components of first  vector: ");
  for(i=0; i<n; i++){
    scanf(SCANFORMAT, &a.x[i]);
  }

  printf("Enter components of second vector: ");
  for(i=0; i<n; i++){
    scanf(SCANFORMAT, &b.x[i]);
  }

  /*----------------------------------------
     calculate dot product
    ----------------------------------------*/
  c = dp(n, a.x, b.x);

  /*----------------------------------------
     print warning if magnitude of
     result is less than 1.0e-6
    ----------------------------------------*/
  if(ABS(c) < 1.0e-6){
    printf("----------------------------------------\n");
    printf("Warning: dot product is less than 1.0e-6\n");
    printf("----------------------------------------\n");
  }

  /*----------------------------------------
     write result to file
    ----------------------------------------*/
  fp = fopen("dot.d", "wb");
  fwrite(&c, sizeof(REAL), 1, fp);
  fclose(fp);

  /*----------------------------------------
     check result by giving c a silly
     value, then reading it from file
    ----------------------------------------*/
  c = 1238765.;
  fp = fopen("dot.d", "rb");
  fread(&c, sizeof(REAL), 1, fp);
  fclose(fp);
  printf("Dot product = %f\n", c);

  /*----------------------------------------
     print result
    ----------------------------------------*/
  printf("Data written to file dot.d.\n");

}
