#include <RcppGSL.h>
#include <gsl/gsl_multifit.h>
#include <cmath>

extern "C" SEXP FastLM(SEXP ys, SEXP Xs) {

  try {

    RcppGSL::vector<double> y = ys; // create gsl data structures from SEXP
    RcppGSL::matrix<double> X = Xs;

    int n = X.nrow(), k = X.ncol();
    double chisq;

    RcppGSL::vector<double> coef(k); // to hold the coecient vector
    RcppGSL::matrix<double> cov(k,k); // and the covariance matrix

    // the actual t requires working memory we allocate and free
    gsl_multifit_linear_workspace *work = gsl_multifit_linear_alloc (n, k);
    gsl_multifit_linear (X, y, coef, cov, &chisq, work);
    gsl_multifit_linear_free (work);

    // extract the diagonal as a vector view
    gsl_vector_view diag = gsl_matrix_diagonal(cov) ;

    // currently there is not a more direct interface in Rcpp::NumericVector
    // that takes advantage of wrap, so we have to do it in two steps
    Rcpp::NumericVector std_err ; std_err = diag;
    std::transform( std_err.begin(), std_err.end(), std_err.begin(), sqrt);
    Rcpp::List res = Rcpp::List::create(Rcpp::Named("coefficients") = coef,
					Rcpp::Named("stderr") = std_err,
					Rcpp::Named("df") = n - k);

    // free all the GSL vectors and matrices -- as these are really C data structures
    // we cannot take advantage of automatic memory management
    coef.free(); cov.free(); y.free(); X.free();

    return res; // return the result list to R

  } catch( std::exception &ex ) {

    forward_exception_to_r( ex );

  } catch(...) {
    ::Rf_error( "c++ exception (unknown reason)" );
  }
  return R_NilValue; 
}
