#include <R.h>
#include <Rdefines.h>
#include <Rmath.h>
#include <math.h>

double log_dens (double x1, double x2, double nu) {
  double a = (x1 + x2) /( nu + 2.);
  return ( log(nu) + log(nu+1) - (nu+2) * log(exp(a) + exp(a-x1) + exp(a-x2)) );
}

SEXP mcmcC (SEXP n){
  double nu = 1.0;
  int i;
  double cur_x[2] = {0.0, 0.0};
  double new_x[2];
  double cur_logf = log_dens(cur_x[0], cur_x[1], nu);
  double new_logf;
  int n_accepted = 0;
  double prob_acceptance;
  double sigma = sqrt(9.0);

  int *np;
  PROTECT(n = AS_INTEGER(n)); // make it an int SEXP
  np = INTEGER(n);            // get pointer to int data
  
  SEXP x;                     // new SEXP
  double *xp;
  PROTECT(x = allocMatrix(REALSXP, *np, 2)); // make new 2d matrix SEXP
  xp = REAL(x);                              // get pointer to real data

  Rprintf("Sample size = %d\n", *np);

  GetRNGstate(); // save random # generator seed 
  for (i=0;i<*np;i++){
    new_x[0]= cur_x[0] + sigma * rnorm(0.,1.); 
    new_x[1]= cur_x[1] + sigma * rnorm(0.,1.); 

    new_logf = log_dens(new_x[0], new_x[1], nu);
    prob_acceptance = exp( new_logf - cur_logf);
    if (unif_rand() < prob_acceptance){
      n_accepted++;
      cur_x[0] = new_x[0]; 
      cur_x[1] = new_x[1];
      cur_logf = new_logf;
    }
    *(xp+i)     = cur_x[0]; // column-major order
    *(xp+i+*np) = cur_x[1]; // 
  }
  PutRNGstate(); // restore random # generator seed 
  UNPROTECT(2);

  Rprintf("Acceptance = %5.2f%%\n", 100.0*n_accepted/(*np));
  return x;
}

