#include <stdio.h>
#include <stdlib.h>
#define MATHLIB_STANDALONE
#include <Rmath.h>
#include <math.h>

double log_dens (double x1, double x2, double nu) {
  double a = (x1 + x2) /( nu + 2.);
  return ( log(nu) + log(nu+1) - (nu+2) * log(exp(a) + exp(a-x1) + exp(a-x2)) );
}

double *mcmcCS (int n){
  double nu = 1.0;
  int i;
  double *x = malloc(sizeof(double)*2*n);
  double cur_x[2] = {0.0, 0.0};
  double new_x[2];
  double cur_logf = log_dens(cur_x[0], cur_x[1], nu);
  double new_logf;
  int n_accepted = 0;
  double prob_acceptance;
  double sigma = sqrt(9.0);

  fprintf(stderr, "Sample size = %d\n", n);

  for (i=0;i<n;i++){
    new_x[0]= cur_x[0] + sigma * rnorm(0.0,1.0); 
    new_x[1]= cur_x[1] + sigma * rnorm(0.0,1.0); 

    new_logf = log_dens(new_x[0], new_x[1], nu);
    prob_acceptance = exp( new_logf - cur_logf);
    if (unif_rand() < prob_acceptance){
      n_accepted++;
      cur_x[0] = new_x[0]; 
      cur_x[1] = new_x[1];
      cur_logf = new_logf;
    }
    *(x+i)   = cur_x[0]; // column-major order
    *(x+i+n) = cur_x[1]; // (for eventual use in R)
  }
  fprintf(stderr, "Acceptance = %5.2f%%\n", 100.0*n_accepted/n);
  return x;
}

int main (int argc, char *argv[]) {
  int n;
  int i;
  double *result;
  if (argc !=2) {
    fprintf(stderr, "Usage: %s sample_size\n", argv[0]);
    exit(1);
  }
  n = atoi(argv[1]);
  result = mcmcCS(n);
  for(i=0;i<n;i++) {
    printf("%5.3f %5.3f\n", *result, *(result+n));
    result++;
  }
}
