log.dens <- function (x1, x2, nu ){ 
   a<- (x1+x2) /(nu+2)
   log(nu) + log(nu+1) - (nu+2) * log(exp(a) + exp(a - x1) + exp(a - x2))
}

mcmcR <- function (n) {
  nu <- 1

  x <- matrix(nrow=n,ncol=2)                     # create matrix to hold sample
  cur.x <- c(0,0)                                # set the initial value
  cur.logf <- log.dens(cur.x[1],cur.x[2],nu=nu)  # evaluate density at cur.x
  n.accepted <- 0                                # counter of the accepted values
  sigma <- sqrt(9)

  cat("Sample size =", n, "\n")

  for (i in 1:n) {                               # repeat n times ...
    new.x <- cur.x + sigma * rnorm(2,0,1)        # create proposed value
    new.logf <- log.dens(new.x[1],new.x[2],nu=nu)

                                        # evaluate density at proposed value
    prob.acceptance <- exp(new.logf-cur.logf) # probability of acceptance
    if (runif(1)<prob.acceptance) {   # if we accept the proposed value ...
      n.accepted <- n.accepted+1 # ... increment the counter of accepted values}
      cur.x <- new.x                    # ... accept the new value
      cur.logf <- new.logf   # ... retain the density at the accepted value
    }

    x[i,] <- cur.x                      # store current value
  }
  cat("Acceptance =", 100.0*n.accepted/n, "\n")
  x
}
