
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define K   2   // number of variables 
#define EPSILON  0.000001


void findJacobian( double *X, double H[2][2]){

	H[0][0] = 2.* X[0];
	H[0][1] = 2.* X[1];
	H[1][0] = -2*X[0]/3.;
	H[1][1] = 1.;

	return;
}

void getF(double *X, double *F){
	F[0] = X[0]*X[0] + X[1]*X[1] - 4.;
	F[1] = -X[0] * X[0]/3. + X[1] ;
	return;
}

// inverse Jacobian matrix
void matrixInverse( double H[K][K], double HT[K][K]){

	int i, j, k;
	double temp;

	// create identity matrix
	for(i = 0; i < K; i++){		
		for(j = 0; j < K; j++ ){	
			if(i == j)									
				HT[i][j] = 1;								
			else											
				HT[i][j] = 0;									
		}
	}

	// start Gauss-Jordan Elimination process
	for(k = 0 ; k < K; k++){
		temp = H[k][k];
		if (abs(temp) < EPSILON) {	//inverse matrix can not be found
				printf(" Error funding inverse matrix\n");
				exit(-1);	// exit the program
		}	
		for(j = 0; j < K; j++){
			H[k][j]/=temp;
			HT[k][j]/=temp;
		}
		for(i = 0; i < K; i++){						
			temp = H[i][k];									 
			for(j = 0; j< K; j++)							 
			{												 
				if(i == k) break;									 
				H[i][j] -= H[k][j] * temp;						 
				HT[i][j] -= HT[k][j] * temp;						 
			}
		}

	}
	return;
}

void matrixVectMult( double HT[K][K], double *F, double *delta){

	int i,j;

	for (i = 0; i < K; i++){
		delta[i] = 0;
		for (j = 0; j < K; j++){
			delta[i] += HT[i][j]*F[j];
		}
	}
	return;
}

int isConverge(double *delta){
	double p = 0;
	int i;

	for( i = 0; i < K; i++) p += delta[i] * delta[i];
	return ( pow(p,0.5) < EPSILON);

}

int main(int argc, char **argv){

int i; 
double H[K][K];		// Jacobian matrix
double HT[K][K];	// Inverse Jacobian matrix

double F[K];			// Function vector
double delta[K];			// step vector
double X[K] = {30,30};		// Solution vector
int n_steps = 0;		// number of steps
int maxSteps = 1000;	// maximum number of steps (to avoid infinite loop )


	do {

		printf(" Current location: %f %f  step # %d \n", X[0], X[1], n_steps);

		findJacobian(X,H);			// Jacobian matrix
		matrixInverse(H, HT);		// Inverse Jacobian matrix
		getF(X, F);					// Find the value of the function in the new location
		matrixVectMult( HT, F, delta);		// multiply inverse Jacobian matrix by the function vector

		// move to the next location
		for ( i=0; i < K; i++) X[i] -= delta[i];

		n_steps++;				// count the number of steps
		if (n_steps == maxSteps) break;

	}while ( !isConverge(delta) );		// check if solution is found

	// print the solution
	printf(" Solution: %f %f is found in %d steps\n", X[0], X[1], n_steps);

	return 0;
}
